@@ -6,6 +6,15 @@
 #include <stdlib.h>
 #include <stdarg.h>
 
+#if defined(HAVE_CODEMEM_VIRTUALALLOC)
+#include <windows.h>
+  #ifdef ORC_WINAPI_ONLY_APP
+    #define _virtualprotect VirtualProtectFromApp
+  #else
+    #define _virtualprotect VirtualProtect
+  #endif
+#endif
+
 #include <orc/orcprogram.h>
 #include <orc/orcdebug.h>
 #include <orc/orcinternal.h>
@@ -48,20 +57,71 @@ int _orc_compiler_flag_emulate;
 int _orc_compiler_flag_debug;
 int _orc_compiler_flag_randomize;
 
+/* For Windows */
+int _orc_codemem_alignment;
+
 void
 _orc_compiler_init (void)
 {
-  const char *envvar;
+  char *envvar;
+#ifdef HAVE_CODEMEM_VIRTUALALLOC
+  size_t page_size;
+  SYSTEM_INFO info;
+#endif
 
-  envvar = getenv ("ORC_CODE");
+  envvar = _orc_getenv ("ORC_CODE");
   if (envvar != NULL) {
     _orc_compiler_flag_list = strsplit (envvar, ',');
+    free (envvar);
   }
 
   _orc_compiler_flag_backup = orc_compiler_flag_check ("backup");
   _orc_compiler_flag_emulate = orc_compiler_flag_check ("emulate");
   _orc_compiler_flag_debug = orc_compiler_flag_check ("debug");
   _orc_compiler_flag_randomize = orc_compiler_flag_check ("randomize");
+
+  /* 16 bytes alignment by default */
+  _orc_codemem_alignment = 15;
+
+#ifdef HAVE_CODEMEM_VIRTUALALLOC
+  GetNativeSystemInfo(&info);
+  page_size = info.dwPageSize;
+
+  /* Protection attribute change via VirtualProtect will be applied per
+   * page memory unit. So we should split code memory with page aligned range.
+   * Otherwise the protection attribute of previously generated executable code
+   * memory can be affected by later generated one.
+   */
+  _orc_codemem_alignment = info.dwPageSize - 1;
+#endif
+
+#ifdef ORC_WINAPI_ONLY_APP
+  if (!_orc_compiler_flag_backup && !_orc_compiler_flag_emulate) {
+    int can_jit = FALSE;
+
+    /* If backup code is not enabled and emulation is not enabled, that means
+     * we will do JIT compilation and call orc_code_region_allocate_codemem().
+     * When targeting Windows Store apps, the codeGeneration capability must
+     * be enabled in the app manifest, or passing PAGE_EXECUTE to
+     * VirtualProtectFromApp will return NULL. In this case, we must force
+     * backup C code, and if that's not available, we must emulate. */
+    void *mem = VirtualAllocFromApp (NULL, page_size, MEM_COMMIT,
+        PAGE_READWRITE);
+    if (mem) {
+      int old_protect;
+      if (VirtualProtectFromApp (mem, page_size, PAGE_EXECUTE, &old_protect) > 0)
+        can_jit = TRUE;
+      VirtualFree (mem, 0, MEM_RELEASE);
+    }
+
+    if (!can_jit) {
+      ORC_WARNING ("Unable to allocate executable pages: using backup code or "
+        "emulation: codeGeneration capability isn't set in the app manifest?");
+      _orc_compiler_flag_backup = TRUE;
+      _orc_compiler_flag_emulate = TRUE;
+    }
+  }
+#endif
 }
 
 int
@@ -174,6 +234,34 @@ orc_program_compile_for_target (OrcProgram *program, OrcTarget *target)
   return orc_program_compile_full (program, target, flags);
 }
 
+#if defined(HAVE_CODEMEM_VIRTUALALLOC)
+static orc_bool
+_set_virtual_protect (void * mem, size_t size, int code_protect)
+{
+  char *msg;
+  DWORD old_protect;
+
+  /* No code, so we 'succeed' */
+  if (size == 0)
+    return TRUE;
+
+  if (!mem)
+    return FALSE;
+
+  if (_virtualprotect (mem, size, code_protect, &old_protect) > 0)
+    return TRUE;
+
+  FormatMessageA (FORMAT_MESSAGE_ALLOCATE_BUFFER
+      | FORMAT_MESSAGE_IGNORE_INSERTS | FORMAT_MESSAGE_FROM_SYSTEM, NULL,
+      GetLastError (), 0, (LPTSTR) &msg, 0, NULL);
+  ORC_ERROR ("Couldn't set memory protect on %p from %x to %x: %s", mem,
+      old_protect, code_protect, msg);
+  LocalFree (msg);
+
+  return FALSE;
+}
+#endif
+
 /**
  * orc_program_compile_full:
  * @program: the OrcProgram to compile
@@ -359,6 +447,11 @@ orc_program_compile_full (OrcProgram *program, OrcTarget *target,
   program->orccode->code_size = compiler->codeptr - compiler->code;
   orc_code_allocate_codemem (program->orccode, program->orccode->code_size);
 
+#if defined(HAVE_CODEMEM_VIRTUALALLOC)
+  /* Ensure that code region is writable before memcpy */
+  _set_virtual_protect (program->orccode->code, program->orccode->code_size,
+      PAGE_READWRITE);
+#endif
   memcpy (program->orccode->code, compiler->code, program->orccode->code_size);
 
 #ifdef VALGRIND_DISCARD_TRANSLATIONS
@@ -370,6 +463,13 @@ orc_program_compile_full (OrcProgram *program, OrcTarget *target,
     compiler->target->flush_cache (program->orccode);
   }
 
+#if defined(HAVE_CODEMEM_VIRTUALALLOC)
+  /* Code region is now ready for execution */
+ if (!_set_virtual_protect (program->orccode->exec, program->orccode->code_size,
+       PAGE_EXECUTE))
+   /* Can't set code as executable, force emulation */
+   program->orccode->exec = (void *)orc_executor_emulate;
+#endif
   program->code_exec = program->orccode->exec;
 
   program->asm_code = compiler->asm_code;
@@ -637,7 +737,7 @@ orc_compiler_rewrite_insns (OrcCompiler *compiler)
         var = compiler->vars + insn.dest_args[i];
         if (var->vartype == ORC_VAR_TYPE_DEST) {
           OrcInstruction *cinsn;
-          
+
           cinsn = compiler->insns + compiler->n_insns;
           compiler->n_insns++;
 
