@@ -15,26 +15,6 @@
 
 /* rules */
 
-static void
-powerpc_load_align (OrcCompiler *compiler, int vector_reg, int offset_reg, int src_reg)
-{
-  if (IS_POWERPC_BE (compiler)) {
-    ORC_ASM_CODE(compiler,"  lvsl %s, %s, %s\n",
-      powerpc_get_regname (vector_reg),
-      offset_reg == 0 ? "0" : powerpc_get_regname (offset_reg),
-      powerpc_get_regname (src_reg));
-    powerpc_emit_X (compiler, 0x7c00000c, powerpc_regnum(vector_reg),
-      offset_reg == 0 ? 0 : powerpc_regnum(offset_reg), powerpc_regnum(src_reg));
-  } else {
-    ORC_ASM_CODE(compiler,"  lvsr %s, %s, %s\n",
-      powerpc_get_regname (vector_reg),
-      offset_reg == 0 ? "0" : powerpc_get_regname (offset_reg),
-      powerpc_get_regname (src_reg));
-    powerpc_emit_X (compiler, 0x7c00004c, powerpc_regnum(vector_reg),
-      offset_reg == 0 ? 0 : powerpc_regnum(offset_reg), powerpc_regnum(src_reg));
-  }
-}
-
 static inline int
 powerpc_select_value (OrcCompiler *compiler, int be_value, int le_value)
 {
@@ -322,19 +302,7 @@ powerpc_rule_storeX (OrcCompiler *compiler, void *user, OrcInstruction *insn)
   int perm = orc_compiler_get_temp_reg (compiler);
   int tmp = orc_compiler_get_temp_reg (compiler);
 
-  if (IS_POWERPC_BE (compiler)) {
-    ORC_ASM_CODE(compiler,"  lvsr %s, 0, %s\n",
-        powerpc_get_regname (perm),
-        powerpc_get_regname (dest->ptr_register));
-    powerpc_emit_X (compiler, 0x7c00004c, powerpc_regnum(perm),
-        0, powerpc_regnum(dest->ptr_register));
-  } else {
-    ORC_ASM_CODE(compiler,"  lvsl %s, 0, %s\n",
-        powerpc_get_regname (perm),
-        powerpc_get_regname (dest->ptr_register));
-    powerpc_emit_X (compiler, 0x7c00000c, powerpc_regnum(perm),
-        0, powerpc_regnum(dest->ptr_register));
-  }
+  powerpc_store_align (compiler, perm, 0, dest->ptr_register);
   powerpc_emit_vperm (compiler, tmp, src->alloc, src->alloc, perm);
 
   switch (size) {
@@ -370,8 +338,7 @@ powerpc_rule_storeX (OrcCompiler *compiler, void *user, OrcInstruction *insn)
           powerpc_regnum(tmp),
           0, powerpc_regnum(dest->ptr_register));
 
-      powerpc_emit_D (compiler, "addi", 0x38000000, compiler->gp_tmpreg,
-          0, 4);
+      powerpc_emit_addi (compiler, compiler->gp_tmpreg, 0, 4);
 
       ORC_ASM_CODE(compiler,"  stvewx %s, %s, %s\n",
           powerpc_get_regname (tmp),
@@ -397,6 +364,48 @@ powerpc_rule_storeX (OrcCompiler *compiler, void *user, OrcInstruction *insn)
   }
 }
 
+static void
+powerpc_denormalize_sp_full(OrcCompiler *p, int reg, int zero, int mask)
+{
+  int tmp = p->tmpreg;
+  powerpc_emit_VX_2(p, "xxland", 0xf0000417, tmp, reg, mask);
+  powerpc_emit_VX_2(p, "vcmpequw", 0x10000086, tmp, tmp, zero);
+  powerpc_emit_VA(p, "xxsel", 0xf000003f, tmp, reg, zero, tmp);
+  powerpc_emit_VX_2(p, "xvcpsgnsp", 0xf0000687, reg, reg, tmp);
+}
+
+static void
+powerpc_denormalize_sp_zero(OrcCompiler *p, int reg, int zero)
+{
+  int mask = powerpc_get_constant_full (p, 0x7f800000, 0x7f800000,
+      0x7f800000, 0x7f800000);
+  powerpc_denormalize_sp_full(p, reg, zero, mask);
+}
+
+static void
+powerpc_denormalize_sp(OrcCompiler *p, int reg)
+{
+  int zero = powerpc_get_constant (p, ORC_CONST_ZERO, 0);
+  powerpc_denormalize_sp_zero(p, reg, zero);
+}
+
+static void
+powerpc_denormalize_dp_full(OrcCompiler *p, int reg, int zero, int mask)
+{
+  int tmp = p->tmpreg;
+  powerpc_emit_VX_2(p, "xxland", 0xf0000417, tmp, reg, mask);
+  powerpc_emit_VX_2(p, "vcmpequd", 0x100000c7, tmp, tmp, zero);
+  powerpc_emit_VA(p, "xxsel", 0xf000003f, tmp, reg, zero, tmp);
+  powerpc_emit_VX_2(p, "xvcpsgndp", 0xf0000787, reg, reg, tmp);
+}
+
+static void
+powerpc_denormalize_dp_zero(OrcCompiler *p, int reg, int zero)
+{
+  int mask = powerpc_get_constant_full (p, 0x7ff00000, 0x00000000,
+      0x7ff00000, 0x00000000);
+  powerpc_denormalize_dp_full(p, reg, zero, mask);
+}
 
 
 #define RULE(name, opcode, code) \
@@ -409,6 +418,33 @@ powerpc_rule_ ## name (OrcCompiler *p, void *user, OrcInstruction *insn) \
   powerpc_emit_VX_2 (p, opcode, code , dest, src1, src2);\
 }
 
+#define RULE_DP(name, opcode, code) \
+static void \
+powerpc_rule_ ## name (OrcCompiler *p, void *user, OrcInstruction *insn) \
+{ \
+  int src1 = ORC_SRC_ARG (p, insn, 0); \
+  int src2 = ORC_SRC_ARG (p, insn, 1); \
+  int dest = ORC_DEST_ARG (p, insn, 0); \
+  int zero = powerpc_get_constant (p, ORC_CONST_ZERO, 0); \
+  powerpc_denormalize_dp_zero (p, src1, zero); \
+  powerpc_denormalize_dp_zero (p, src2, zero); \
+  powerpc_emit_VX_2 (p, opcode, code , dest, src1, src2); \
+  powerpc_denormalize_dp_zero (p, dest, zero); \
+}
+
+#define RULE_DP_SRC(name, opcode, code) \
+static void \
+powerpc_rule_ ## name (OrcCompiler *p, void *user, OrcInstruction *insn) \
+{ \
+  int src1 = ORC_SRC_ARG (p, insn, 0); \
+  int src2 = ORC_SRC_ARG (p, insn, 1); \
+  int dest = ORC_DEST_ARG (p, insn, 0); \
+  int zero = powerpc_get_constant (p, ORC_CONST_ZERO, 0); \
+  powerpc_denormalize_dp_zero (p, src1, zero); \
+  powerpc_denormalize_dp_zero (p, src2, zero); \
+  powerpc_emit_VX_2 (p, opcode, code , dest, src1, src2); \
+}
+
 #define RULE_SHIFT(name, opcode, code) \
 static void \
 powerpc_rule_ ## name (OrcCompiler *p, void *user, OrcInstruction *insn) \
@@ -503,13 +539,11 @@ RULE(maxf, "vmaxfp", 0x1000040a)
 RULE(minf, "vminfp", 0x1000044a)
 RULE(cmpeqf, "vcmpeqfp", 0x100000c6)
 
-RULE(addd, "xvadddp", 0xf0000307)
-RULE(subd, "xvsubdp", 0xf0000347)
-RULE(muld, "xvmuldp", 0xf0000387)
-RULE(divd, "xvdivdp", 0xf00003c7)
-RULE(mind, "xvmindp", 0xf0000747)
-RULE(maxd, "xvmaxdp", 0xf0000707)
-RULE(cmpeqd, "xvcmpeqdp", 0xf000031f)
+RULE_DP(addd, "xvadddp", 0xf0000307)
+RULE_DP(subd, "xvsubdp", 0xf0000347)
+RULE_DP(muld, "xvmuldp", 0xf0000387)
+RULE_DP(divd, "xvdivdp", 0xf00003c7)
+RULE_DP_SRC(cmpeqd, "xvcmpeqdp", 0xf000031f)
 
 RULE(addq, "vaddudm", 0x100000c0)
 RULE(subq, "vsubudm", 0x100004c0)
@@ -541,13 +575,12 @@ powerpc_rule_mullb (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src1 = ORC_SRC_ARG (p, insn, 0);
   int src2 = ORC_SRC_ARG (p, insn, 1);
   int dest = ORC_DEST_ARG (p, insn, 0);
+  int perm = powerpc_get_constant_full(p, 0x11011303, 0x15051707,
+      0x19091b0b, 0x1d0d1f0f);
 
-  if (IS_POWERPC_BE (p)) {
-    powerpc_emit_vmulesb (p, dest, src1, src2);
-    powerpc_emit_vsldoi (p, dest, dest, dest, 1);
-  } else {
-    powerpc_emit_vmulosb (p, dest, src1, src2);
-  }
+  powerpc_emit_vmulesb (p, p->tmpreg, src1, src2);
+  powerpc_emit_vmulosb (p, dest, src1, src2);
+  powerpc_emit_vperm(p, dest, dest, p->tmpreg, perm);
 }
 
 static void
@@ -556,15 +589,12 @@ powerpc_rule_mulhsb (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src1 = ORC_SRC_ARG (p, insn, 0);
   int src2 = ORC_SRC_ARG (p, insn, 1);
   int dest = ORC_DEST_ARG (p, insn, 0);
+  int perm = powerpc_get_constant_full(p, 0x10001202, 0x14041606,
+      0x18081a0a, 0x1c0c1e0e);
 
-  if (IS_POWERPC_BE (p)) {
-    powerpc_emit_vmulesb (p, dest, src1, src2);
-  } else {
-    int tmp = powerpc_get_constant (p, ORC_CONST_SPLAT_B, 8);
-
-    powerpc_emit_vmulosb (p, dest, src1, src2);
-    powerpc_emit_VX_2 (p, "vsro", 0x1000044c, dest, src1, tmp);
-  }
+  powerpc_emit_vmulesb (p, p->tmpreg, src1, src2);
+  powerpc_emit_vmulosb (p, dest, src1, src2);
+  powerpc_emit_vperm(p, dest, dest, p->tmpreg, perm);
 }
 
 static void
@@ -573,15 +603,12 @@ powerpc_rule_mulhub (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src1 = ORC_SRC_ARG (p, insn, 0);
   int src2 = ORC_SRC_ARG (p, insn, 1);
   int dest = ORC_DEST_ARG (p, insn, 0);
+  int perm = powerpc_get_constant_full(p, 0x10001202, 0x14041606,
+      0x18081a0a, 0x1c0c1e0e);
 
-  if (IS_POWERPC_BE (p)) {
-    powerpc_emit_vmuleub (p, dest, src1, src2);
-  } else {
-    int tmp = powerpc_get_constant (p, ORC_CONST_SPLAT_B, 8);
-
-    powerpc_emit_vmuloub (p, dest, src1, src2);
-    powerpc_emit_VX_2 (p, "vsro", 0x1000044c, dest, dest, tmp);
-  }
+  powerpc_emit_vmuleub (p, p->tmpreg, src1, src2);
+  powerpc_emit_vmuloub (p, dest, src1, src2);
+  powerpc_emit_vperm(p, dest, dest, p->tmpreg, perm);
 }
 
 static void
@@ -590,16 +617,12 @@ powerpc_rule_mulhsw (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src1 = ORC_SRC_ARG (p, insn, 0);
   int src2 = ORC_SRC_ARG (p, insn, 1);
   int dest = ORC_DEST_ARG (p, insn, 0);
+  int perm = powerpc_get_constant_full(p, 0x10110001, 0x14150405,
+      0x18190809, 0x1c1d0c0d);
 
-  if (IS_POWERPC_BE (p)) {
-    powerpc_emit_vmulesh (p, dest, src1, src2);
-  } else {
-    int tmp = powerpc_get_constant (p, ORC_CONST_SPLAT_B, 8);
-
-    powerpc_emit_vmulosh (p, dest, src1, src2);
-    powerpc_emit_VX_2 (p, "vsro", 0x1000044c, dest, dest, tmp);
-    powerpc_emit_VX_2 (p, "vsro", 0x1000044c, dest, dest, tmp);
-  }
+  powerpc_emit_vmulesh (p, p->tmpreg, src1, src2);
+  powerpc_emit_vmulosh (p, dest, src1, src2);
+  powerpc_emit_vperm(p, dest, dest, p->tmpreg, perm);
 }
 
 static void
@@ -608,16 +631,12 @@ powerpc_rule_mulhuw (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src1 = ORC_SRC_ARG (p, insn, 0);
   int src2 = ORC_SRC_ARG (p, insn, 1);
   int dest = ORC_DEST_ARG (p, insn, 0);
+  int perm = powerpc_get_constant_full(p, 0x10110001, 0x14150405,
+      0x18190809, 0x1c1d0c0d);
 
-  if (IS_POWERPC_BE (p)) {
-    powerpc_emit_vmuleuh (p, dest, src1, src2);
-  } else {
-    int tmp = powerpc_get_constant (p, ORC_CONST_SPLAT_B, 8);
-
-    powerpc_emit_vmulouh (p, dest, src1, src2);
-    powerpc_emit_VX_2 (p, "vsro", 0x1000044c, dest, dest, tmp);
-    powerpc_emit_VX_2 (p, "vsro", 0x1000044c, dest, dest, tmp);
-  }
+  powerpc_emit_vmuleuh(p, p->tmpreg, src1, src2);
+  powerpc_emit_vmulouh(p, dest, src1, src2);
+  powerpc_emit_vperm(p, dest, dest, p->tmpreg, perm);
 }
 
 static void
@@ -765,10 +784,12 @@ powerpc_rule_mulsbw (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src2 = ORC_SRC_ARG (p, insn, 1);
   int dest = ORC_DEST_ARG (p, insn, 0);
 
+  powerpc_emit_vmulesb (p, p->tmpreg, src1, src2);
+  powerpc_emit_vmulosb (p, dest, src1, src2);
   if (IS_POWERPC_BE (p)) {
-    powerpc_emit_vmulesb (p, dest, src1, src2);
+    powerpc_emit_vmrghh (p, dest, p->tmpreg, dest);
   } else {
-    powerpc_emit_vmulosb (p, dest, src1, src2);
+    powerpc_emit_vmrglh (p, dest, p->tmpreg, dest);
   }
 }
 
@@ -779,10 +800,12 @@ powerpc_rule_mulubw (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src2 = ORC_SRC_ARG (p, insn, 1);
   int dest = ORC_DEST_ARG (p, insn, 0);
 
+  powerpc_emit_vmuleub (p, p->tmpreg, src1, src2);
+  powerpc_emit_vmuloub (p, dest, src1, src2);
   if (IS_POWERPC_BE (p)) {
-    powerpc_emit_vmuleub (p, dest, src1, src2);
+    powerpc_emit_vmrghh (p, dest, p->tmpreg, dest);
   } else {
-    powerpc_emit_vmuloub (p, dest, src1, src2);
+    powerpc_emit_vmrglh (p, dest, p->tmpreg, dest);
   }
 }
 
@@ -793,10 +816,12 @@ powerpc_rule_mulswl (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src2 = ORC_SRC_ARG (p, insn, 1);
   int dest = ORC_DEST_ARG (p, insn, 0);
 
+  powerpc_emit_vmulesh (p, p->tmpreg, src1, src2);
+  powerpc_emit_vmulosh (p, dest, src1, src2);
   if (IS_POWERPC_BE (p)) {
-    powerpc_emit_vmulesh (p, dest, src1, src2);
+    powerpc_emit_vmrghw (p, dest, p->tmpreg, dest);
   } else {
-    powerpc_emit_vmulosh (p, dest, src1, src2);
+    powerpc_emit_vmrglw (p, dest, p->tmpreg, dest);
   }
 }
 
@@ -807,10 +832,12 @@ powerpc_rule_muluwl (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src2 = ORC_SRC_ARG (p, insn, 1);
   int dest = ORC_DEST_ARG (p, insn, 0);
 
+  powerpc_emit_vmuleuh (p, p->tmpreg, src1, src2);
+  powerpc_emit_vmulouh (p, dest, src1, src2);
   if (IS_POWERPC_BE (p)) {
-    powerpc_emit_vmuleuh (p, dest, src1, src2);
+    powerpc_emit_vmrghw (p, dest, p->tmpreg, dest);
   } else {
-    powerpc_emit_vmulouh (p, dest, src1, src2);
+    powerpc_emit_vmrglw (p, dest, p->tmpreg, dest);
   }
 }
 
@@ -848,17 +875,13 @@ powerpc_rule_mulll (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src1 = ORC_SRC_ARG (p, insn, 0);
   int src2 = ORC_SRC_ARG (p, insn, 1);
   int dest = ORC_DEST_ARG (p, insn, 0);
+  int tmp = orc_compiler_get_temp_reg(p);
+  int perm = powerpc_get_constant_full(p, 0x14151617, 0x04050607,
+      0x1c1d1e1f, 0x0c0d0e0f);
 
-  if (IS_POWERPC_BE (p)) {
-    int perm;
-
-    powerpc_emit_vmuleuw (p, dest, src1, src2);
-    perm = powerpc_get_constant_full (p, 0x04050607, 0x04050607,
-        0x0c0d0e0f, 0x0c0d0e0f);
-    powerpc_emit_vperm (p, dest, dest, dest, perm);
-  } else {
-    powerpc_emit_vmulouw (p, dest, src1, src2);
-  }
+  powerpc_emit_vmulouw (p, tmp, src1, src2);
+  powerpc_emit_vmuleuw (p, dest, src1, src2);
+  powerpc_emit_vperm (p, dest, tmp, dest, perm);
 }
 
 static void
@@ -1428,20 +1451,11 @@ powerpc_rule_divf (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src1 = ORC_SRC_ARG (p, insn, 0);
   int src2 = ORC_SRC_ARG (p, insn, 1);
   int dest = ORC_DEST_ARG (p, insn, 0);
-  int y = orc_compiler_get_temp_reg (p);
-  int t = orc_compiler_get_temp_reg (p);
-  int c1;
-  int c0;
-
-  c1 = powerpc_get_constant (p, ORC_CONST_SPLAT_L, 0x3f800000); /* 1.0 */
-
-  powerpc_emit_VX_db (p, "vrefp", 0x1000010a, y, src2);
-
-  powerpc_emit_VA_acb (p, "vnmsubfp", 0x1000002f, t, y, c1, src2);
-  powerpc_emit_VA_acb (p, "vmaddfp", 0x1000002e, y, y, y, t);
-
-  c0 = powerpc_get_constant (p, ORC_CONST_SPLAT_L, 0x00000000); /* 0.0 */
-  powerpc_emit_VA_acb (p, "vmaddfp", 0x1000002e, dest, y, c0, src1);
+  int zero = powerpc_get_constant (p, ORC_CONST_ZERO, 0);
+  powerpc_denormalize_sp_zero (p, src1, zero);
+  powerpc_denormalize_sp_zero (p, src2, zero);
+  powerpc_emit_VX_2(p, "xvdivsp", 0xf00002c7, dest, src1, src2);
+  powerpc_denormalize_sp_zero (p, dest, zero);
 }
 
 static void
@@ -1521,8 +1535,10 @@ powerpc_rule_sqrtf(OrcCompiler* p, void* user, OrcInstruction* insn)
 {
   int src1 = ORC_SRC_ARG(p, insn, 0);
   int dest = ORC_DEST_ARG(p, insn, 0);
-
+  int zero = powerpc_get_constant (p, ORC_CONST_ZERO, 0);
+  powerpc_denormalize_sp_zero (p, src1, zero);
   powerpc_emit_VX_db(p, "xvsqrtsp", 0xf000022f, dest, src1);
+  powerpc_denormalize_sp_zero (p, dest, zero);
 }
 
 static void
@@ -1530,8 +1546,54 @@ powerpc_rule_sqrtd(OrcCompiler* p, void* user, OrcInstruction* insn)
 {
   int src1 = ORC_SRC_ARG(p, insn, 0);
   int dest = ORC_DEST_ARG(p, insn, 0);
-
+  int zero = powerpc_get_constant (p, ORC_CONST_ZERO, 0);
+  powerpc_denormalize_dp_zero (p, src1, zero);
   powerpc_emit_VX_db(p, "xvsqrtdp", 0xf000032f, dest, src1);
+  powerpc_denormalize_dp_zero (p, dest, zero);
+}
+
+static void
+powerpc_rule_mind (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  int src1 = ORC_SRC_ARG (p, insn, 0);
+  int src2 = ORC_SRC_ARG (p, insn, 1);
+  int dest = ORC_DEST_ARG (p, insn, 0);
+  int zero = powerpc_get_constant (p, ORC_CONST_ZERO, 0);
+  int mask = powerpc_get_constant_full (p, 0x7ff80000, 0x00000000,
+      0x7ff80000, 0x00000000);
+  int tmp2 = orc_compiler_get_temp_reg(p);
+  int tmp = p->tmpreg;
+  powerpc_denormalize_dp_zero (p, src1, zero);
+  powerpc_denormalize_dp_zero (p, src2, zero);
+  powerpc_emit_VX_2(p, "xvmindp", 0xf0000747, tmp2, src2, src1);
+  // Handle QNaN
+  powerpc_emit_VX_2(p, "xxland", 0xf0000417, tmp, src1, mask);
+  powerpc_emit_VX_2(p, "vcmpequd", 0x100000c7, tmp, tmp, mask);
+  powerpc_emit_VA(p, "xxsel", 0xf000003f, dest, tmp2, src1, tmp);
+}
+
+static void
+powerpc_rule_maxd (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  int src1 = ORC_SRC_ARG (p, insn, 0);
+  int src2 = ORC_SRC_ARG (p, insn, 1);
+  int dest = ORC_DEST_ARG (p, insn, 0);
+  int zero = powerpc_get_constant (p, ORC_CONST_ZERO, 0);
+  int mask = powerpc_get_constant_full (p, 0x7ff80000, 0x00000000,
+      0x7ff80000, 0x00000000);
+  int tmp2 = orc_compiler_get_temp_reg(p);
+  int tmp3 = orc_compiler_get_temp_reg(p);
+  int tmp = p->tmpreg;
+  powerpc_denormalize_dp_zero (p, src1, zero);
+  powerpc_denormalize_dp_zero (p, src2, zero);
+  powerpc_emit_VX_2(p, "xvmaxdp", 0xf0000707, tmp2, src2, src1);
+  // Handle QNaN
+  powerpc_emit_VX_2(p, "xxland", 0xf0000417, tmp, src2, mask);
+  powerpc_emit_VX_2(p, "vcmpequd", 0x100000c7, tmp, tmp, mask);
+  powerpc_emit_VA(p, "xxsel", 0xf000003f, tmp3, tmp2, src2, tmp);
+  powerpc_emit_VX_2(p, "xxland", 0xf0000417, tmp, src1, mask);
+  powerpc_emit_VX_2(p, "vcmpequd", 0x100000c7, tmp, tmp, mask);
+  powerpc_emit_VA(p, "xxsel", 0xf000003f, dest, tmp3, src1, tmp);
 }
 
 static void
@@ -1540,8 +1602,9 @@ powerpc_rule_cmpltd (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src1 = ORC_SRC_ARG (p, insn, 0);
   int src2 = ORC_SRC_ARG (p, insn, 1);
   int dest = ORC_DEST_ARG (p, insn, 0);
-
-//  powerpc_emit_VXR (p, "vcmpgtfp", 0x100002c6, dest, src2, src1, FALSE);
+  int zero = powerpc_get_constant (p, ORC_CONST_ZERO, 0);
+  powerpc_denormalize_dp_zero (p, src1, zero);
+  powerpc_denormalize_dp_zero (p, src2, zero);
   powerpc_emit_VX_2(p, "xvcmpgtdp", 0xf000035f, dest, src2, src1);
 }
 
@@ -1551,8 +1614,9 @@ powerpc_rule_cmpled (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src1 = ORC_SRC_ARG (p, insn, 0);
   int src2 = ORC_SRC_ARG (p, insn, 1);
   int dest = ORC_DEST_ARG (p, insn, 0);
-
-//  powerpc_emit_VXR (p, "vcmpgefp", 0x100001c6, dest, src2, src1, FALSE);
+  int zero = powerpc_get_constant (p, ORC_CONST_ZERO, 0);
+  powerpc_denormalize_dp_zero (p, src1, zero);
+  powerpc_denormalize_dp_zero (p, src2, zero);
   powerpc_emit_VX_2(p, "xvcmpgedp", 0xf000039f, dest, src2, src1);
 }
 
@@ -1561,11 +1625,13 @@ powerpc_rule_convld (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
   int src1 = ORC_SRC_ARG (p, insn, 0);
   int dest = ORC_DEST_ARG (p, insn, 0);
+  int zero = powerpc_get_constant (p, ORC_CONST_ZERO, 0);
 
   if (IS_POWERPC_LE (p)) {
     powerpc_emit_vsldoi(p, src1, src1, src1, 4);
   }
   powerpc_emit_VX_db (p, "xvcvsxddp", 0xf00003e3, dest, src1);
+  powerpc_denormalize_dp_zero (p, dest, zero);
 }
 
 static void
@@ -1573,8 +1639,19 @@ powerpc_rule_convdl (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
   int src1 = ORC_SRC_ARG (p, insn, 0);
   int dest = ORC_DEST_ARG (p, insn, 0);
+  int mask = powerpc_get_constant_full (p, 0xfff00000, 0x00000000,
+      0xfff00000, 0x00000000);
+  int nan = powerpc_get_constant_full (p, 0x7ff00000, 0x00000000,
+      0x7ff00000, 0x00000000);
+  int maxint = powerpc_get_constant_full (p, 0x7fffffff, 0x7fffffff,
+      0x7fffffff, 0x7fffffff);
+  int tmp = p->tmpreg;
 
+  powerpc_emit_VX_2(p, "xxland", 0xf0000417, tmp, src1, mask);
+  powerpc_emit_VX_2(p, "vcmpequd", 0x100000c7, tmp, tmp, nan);
   powerpc_emit_VX_db (p, "xvcvdpsxws", 0xf0000363, dest, src1);
+  powerpc_emit_VA(p, "xxsel", 0xf000003f, dest, dest, maxint, tmp);
+
   if (IS_POWERPC_LE (p)) {
     int tmp = powerpc_get_constant (p, ORC_CONST_SPLAT_B, 32);
     powerpc_emit_VX_2 (p, "vsro", 0x1000044c, dest, dest, tmp);
@@ -1586,11 +1663,23 @@ powerpc_rule_convfd (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
   int src1 = ORC_SRC_ARG (p, insn, 0);
   int dest = ORC_DEST_ARG (p, insn, 0);
+  int tmp2 = orc_compiler_get_temp_reg(p);
+  int zero = powerpc_get_constant (p, ORC_CONST_ZERO, 0);
+  int mask = powerpc_get_constant_full (p, 0x7f800000, 0x00000000,
+      0x7f800000, 0x00000000);
+  int tmp = p->tmpreg;
 
   if (IS_POWERPC_LE (p)) {
-    powerpc_emit_vsldoi(p, src1, src1, src1, 4);
+    powerpc_emit_vsldoi(p, tmp2, src1, src1, 4);
+  } else {
+    powerpc_emit_vor(p, tmp2, src1, src1);
   }
-  powerpc_emit_VX_db (p, "xvcvspdp", 0xf0000727, dest, src1);
+  powerpc_emit_VX_db (p, "xvcvspdp", 0xf0000727, dest, tmp2);
+
+  powerpc_emit_VX_2(p, "xxland", 0xf0000417, tmp, tmp2, mask);
+  powerpc_emit_VX_2(p, "vcmpequd", 0x100000c7, tmp, tmp, zero);
+  powerpc_emit_VA(p, "xxsel", 0xf000003f, tmp, dest, zero, tmp);
+  powerpc_emit_VX_2(p, "xvcpsgndp", 0xf0000787, dest, dest, tmp);
 }
 
 static void
@@ -1604,6 +1693,7 @@ powerpc_rule_convdf (OrcCompiler *p, void *user, OrcInstruction *insn)
     int tmp = powerpc_get_constant (p, ORC_CONST_SPLAT_B, 32);
     powerpc_emit_VX_2 (p, "vsro", 0x1000044c, dest, dest, tmp);
   }
+  powerpc_denormalize_sp(p, dest);
 }
 
 
@@ -1757,7 +1847,6 @@ orc_compiler_powerpc_register_rules (OrcTarget *target)
   REG(cmplef);
   REG(cmpltf);
   REG(mulf);
-  REG(divf);
   REG(convfl);
   REG(convlf);
 
@@ -1789,6 +1878,7 @@ orc_compiler_powerpc_register_rules (OrcTarget *target)
 
   rule_set = orc_rule_set_new(orc_opcode_set_get("sys"), target, ORC_TARGET_POWERPC_VSX);
 
+  REG(divf);
   REG(sqrtf);
   REG(addd);
   REG(subd);
