@@ -31,7 +31,11 @@ main (int argc, char *argv[])
   orc_init ();
   orc_test_init ();
 
+#ifdef ORC_TEST_FILENAME
+  filename = ORC_TEST_FILENAME;
+#else
   filename = "bench10.orc";
+#endif
   code = read_file (filename);
   if (!code) {
     printf("benchmorc needs bench10.orc file in current directory\n");
@@ -89,7 +93,7 @@ read_file (const char *filename)
   long size;
   int ret;
 
-  file = fopen (filename, "r");
+  file = fopen (filename, "rb");
   if (file == NULL) return NULL;
 
   ret = fseek (file, 0, SEEK_END);
